@@ -1,4 +1,4 @@
-// Copyright (C) 2002-2010 Nikolaus Gebhardt
+// Copyright (C) 2002-2012 Nikolaus Gebhardt
 // This file is part of the "Irrlicht Engine".
 // For conditions of distribution and use, see copyright notice in irrlicht.h
 
@@ -18,7 +18,8 @@ namespace scene
 		EAC_OFF = 0,
 		EAC_BOX = 1,
 		EAC_FRUSTUM_BOX = 2,
-		EAC_FRUSTUM_SPHERE = 4
+		EAC_FRUSTUM_SPHERE = 4,
+		EAC_OCC_QUERY = 8
 	};
 
 	//! Names for culling type
@@ -28,6 +29,7 @@ namespace scene
 		"box",			// camera box against node box
 		"frustum_box",		// camera frustum against node box
 		"frustum_sphere",	// camera frustum against node sphere
+		"occ_query",	// occlusion query
 		0
 	};
 
